@@ -1,9 +1,9 @@
 /*
  * (C) Copyright 1996-2016 ECMWF.
- * 
+ *
  * This software is licensed under the terms of the Apache Licence Version 2.0
- * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
- * In applying this licence, ECMWF does not waive the privileges and immunities 
+ * which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
+ * In applying this licence, ECMWF does not waive the privileges and immunities
  * granted to it by virtue of its status as an intergovernmental organisation nor
  * does it submit to any jurisdiction.
  */
@@ -20,7 +20,6 @@
 */
 
 
-#include "magics_ecmwf_log.h"
 #include "FortranMagics.h"
 #include "Timer.h"
 
@@ -79,8 +78,6 @@ FortranMagics::FortranMagics() :  drivers_(0), output_(0), action_(0), empty_(tr
 {
 	ASSERT (singleton_ == 0);
 	singleton_ = this;
-
-	writeMagLog("fortran");
 }
 
 
@@ -139,33 +136,41 @@ void FortranMagics::popen()
   Here is where the real magics is happen. Everything is dispatched, followed
   by a comprehensive clean-up.
 */
-void FortranMagics::pclose()
+int FortranMagics::pclose()
 {
 	MagLog::info()<< "pclose()" << endl;
-	if (!empty_) {
-		finish();
-		dispatch();
-	}
-
-	if ( root_ && drivers_ ) {
-
-		BasicGraphicsObject* object = root_->close();
-		if ( object ) {
-
-			/***   Start clean-up  ***/
-			drivers_->dispatch(object);
-			drivers_->closeDrivers();
-
-			delete root_;
-			delete drivers_;
-			delete output_;
-
-			drivers_ = 0;
-			root_ = 0;
-			output_ = 0;
+	try {
+		if (!empty_) {
+			finish();
+			dispatch();
 		}
 
+		if ( root_ && drivers_ ) {
+
+			BasicGraphicsObject* object = root_->close();
+			if ( object ) {
+
+			/***   Start clean-up  ***/
+				drivers_->dispatch(object);
+				drivers_->closeDrivers();
+
+				delete root_;
+				delete drivers_;
+				delete output_;
+
+				drivers_ = 0;
+				root_ = 0;
+				output_ = 0;
+			}
+
+		}
 	}
+	catch ( MagicsException e) {
+		MagLog::error() << "Errors reported:" << e.what() << " - No plot produced  " << endl;
+		MagLog::error().flush();
+		return -1;
+	}
+
 
 	// the Magics log messages are not broadcast until the next log event - therefore, the
 	// last log message will not be broadcast. We fix that by flushing the message streams
@@ -176,6 +181,7 @@ void FortranMagics::pclose()
 	// then a consecutive call to popen will not be affected by the current values.
 	ParameterManager::reset();
 
+	Layout::reset();
 
 	if(getEnvVariable("MAGPLUS_QUIET").empty() )
 	{
@@ -188,6 +194,7 @@ void FortranMagics::pclose()
 		{
 			MagLog::userInfo() << "  - "<<(*it)<<"\n";
 		}
+
 		MagLog::userInfo() << "\n";
 	*/
 		MagLog::userInfo() << "------------------------------------------------------------------\n";
@@ -197,6 +204,7 @@ void FortranMagics::pclose()
 		MagLog::userInfo() << "                   magics@ecmwf.int\n";
 		MagLog::userInfo() << "------------------------------------------------------------------\n";
 	}
+	return 0;
 }
 
 void FortranMagics::drivers()
@@ -829,7 +837,7 @@ void FortranMagics::pline()
 	actions();
 
 
-	if ( !action_ ) {
+	if ( !action_ || polyinput_todo_ ) {
 		action_ = new VisualAction();
 
 		polyinput_todo_ = false;
@@ -945,6 +953,17 @@ void FortranMagics::epsgraph()
 	action_->visdef(epsgraph);
 }
 
+void FortranMagics::epslight()
+{
+	actions();
+	if ( !action_ ) {
+			MagLog::error() << "epslight-> No data defined " << endl;
+			exit(1);
+		}
+	EpsLight* epslight = new EpsLight();
+	action_->visdef(epslight);
+}
+
 void FortranMagics::epswave()
 {
 	actions();
